#!/usr/bin/env python
# -----------------------------------------------------------------------------
# Script developed by A. Saraiva-Souza, research scientist
# Description: Generates a (001) TiO₂ anatase slab from a CIF file,
#              trims it to a target thickness, applies vacuum, recenters
#              along the z-axis, and exports a 2x2x1 supercell to CIF format.
# -----------------------------------------------------------------------------

from pymatgen.io.cif import CifParser, CifWriter
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from pymatgen.core.surface import SlabGenerator
from pymatgen.core.structure import Structure
import numpy as np
import warnings
warnings.filterwarnings("ignore", category=DeprecationWarning)

# === 1. Read structure from CIF file ===
parser = CifParser("TiO2_anatase_I41amd.cif")
structure = parser.parse_structures(primitive=False)[0]

# === 2. Get conventional cell ===
sga = SpacegroupAnalyzer(structure)
conventional_structure = sga.get_conventional_standard_structure()

# === 3. Generate (001) slab with sufficient minimum thickness
slabgen = SlabGenerator(
    conventional_structure,
    miller_index=(0, 0, 1),
    min_slab_size=12.0,   
    min_vacuum_size=15.0,
    center_slab=True,
    primitive=False
)

# === 4. Get orthogonal slab
slab = slabgen.get_slabs()[0].get_orthogonal_c_slab()

# === 5. Adjusting the slab thickness
target_thickness = 12.0  # Å
coords_z = np.array([site.coords[2] for site in slab])
z_center = (coords_z.max() + coords_z.min()) / 2
z_min = z_center - target_thickness / 2
z_max = z_center + target_thickness / 2

# Filter atoms within the desired z range
sites_in_range = [site for site in slab if z_min <= site.coords[2] <= z_max]

# Create a temporary structure with the trimmed atoms
new_slab = Structure(
    lattice=slab.lattice,
    species=[site.specie for site in sites_in_range],
    coords=[site.frac_coords for site in sites_in_range],
    coords_are_cartesian=False
)

# === 6. Adjust c vector for new height 
vacuum = 15.0
new_c = target_thickness + vacuum
c_dir = slab.lattice.matrix[2] / np.linalg.norm(slab.lattice.matrix[2])
new_c_vector = c_dir * new_c
new_lattice = slab.lattice.matrix.copy()
new_lattice[2] = new_c_vector

# Recentering the slab along the z-axis
z_shift = (vacuum / 2) - (z_min)
new_cart_coords = np.array([site.coords for site in new_slab]) + np.array([0, 0, z_shift])

# Create the final structure with adjusted cell and shifted atoms
final_slab = Structure(
    lattice=new_lattice,
    species=[site.specie for site in new_slab],
    coords=new_cart_coords,
    coords_are_cartesian=True
)

# === 7. Expand to 2x2x1 supercell
final_slab.make_supercell([[2, 0, 0], [0, 2, 0], [0, 0, 1]])

# === 8. Save CIF file
CifWriter(final_slab, symprec=None, refine_struct=False).write_file("TiO2_anatase_001_slab_2x2x1.cif")

# === 9. Final message
print("✅ Slab saved as TiO2_anatase_001_slab_2x2x1.cif")
